from pathlib import Path

home = Path.home()
files = {
    'history': f'{home}/span/history',
    'instances': f'{home}/span/instances',
    'key': f'{home}/span/key',
    'account': f'{home}/span/account'
}
